@@ -19,6 +19,7 @@ cv.gamlr <- function(x, y, nfold=5, foldid=NULL, verb=FALSE, cl=NULL, ...){
   nfold <- nlevels(foldid)
 
   argl <- list(...)
+  if(!is.null(argl$fix)) fix <- argl$fix
   lambda <- as.double(full$lambda)
   argl$lambda.start <- lambda[1]
   argl$nlambda <- length(lambda)
@@ -36,9 +37,11 @@ cv.gamlr <- function(x, y, nfold=5, foldid=NULL, verb=FALSE, cl=NULL, ...){
   folddev <- function(k){
     require(gamlr)
     train <- which(foldid!=k)
+    if(!is.null(argl$fix)) argl$fix <- fix[train]
     suppressWarnings(fit <- do.call(gamlr, 
       c(list(x=x[train,],y=y[train]), argl)))
     eta <- predict(fit, x[-train,], select=0)
+    if(!is.null(argl$fix)) eta <- eta + fix[-train]
 
     dev <- apply(eta,2, 
       function(e) 
@@ -70,8 +73,7 @@ cv.gamlr <- function(x, y, nfold=5, foldid=NULL, verb=FALSE, cl=NULL, ...){
       cl <- NULL
     }
   }
-  if(is.null(cl)) 
-    oos <- t(sapply(1:nfold,folddev))
+  if(is.null(cl)) oos <- t(sapply(1:nfold,folddev))
 
   cvm <- apply(oos,2,mean)
   cvs <- apply(oos,2,sd)/sqrt(nfold-1)
